function runChoiceImplementation(varargin)
%-------------------------------------------------------------------------%
% Script for selecting a single trial from among all the choice trials the
% participant saw, and displaying the resulting outcome. Note that, because
% foods can sometimes run out, we keep a "FoodTracking.xlsx" file on hand
% that lists available quantities of food. Assuming that research
% assistants are diligent about keeping this file up to date, this ensures 
% that we never select a trial for a participant where we cannot deliver
% the food.
%
% Author: Cendri Hutcherson
% Last modified: 05-27-2018
%-------------------------------------------------------------------------%

%% Initial set up
% Allows two modes: called from main script, or called independently

studyID = 'ExampleStudy'; % this should match the study directory name
defaultSessionID = 'ChoiceImplementation';

if isempty(varargin)
    % If called independently, set up PsychToolBox, add the proper paths,
    % and then create a data file
    
    % Get paths to allow for independence across platforms
    pathToFile = mfilename('fullpath');
    homePath = pathToFile(1:(regexp(pathToFile,studyID) + length(studyID)));
    addpath(fullfile(homePath, 'PTBscripts'))
    addpath(fullfile(homePath, 'PTBscripts', 'PTBToolBox'))
    PTBParams = InitPTB(homePath,'DefaultSession',defaultSessionID);
else
    % If called from runStudy.m, the function will be supplied with the
    % PTBParams variable, which will be used to create a new data file for
    % this task component
    PTBParams = varargin{1};
    
    Data.subjid = PTBParams.subjid;
    Data.ssnid = defaultSessionID;
    Data.startTime = datestr(now);
 
    PTBParams.datafile = fullfile(PTBParams.homepath, 'SubjectData', ...
                         num2str(PTBParams.subjid), ...
                         ['Data.' num2str(PTBParams.subjid) '.' Data.ssnid '.mat']);
    save(PTBParams.datafile, 'Data')
end

pFile = PTBParams.datafile;

SessionClockStart = GetSecs();
logData(pFile,1,SessionClockStart);
PTBParams.SessionClockStart = SessionClockStart;

%% Set info about where to locate images
% All images are stored in a folder called 'ImageFiles' located inside the
% PTBscripts folder
imgpath = fullfile(PTBParams.homepath, 'PTBscripts', 'ImageFiles');

% Define the path to the folder where all the food images are stored
PTBParams.foodPath = fullfile(imgpath, 'FoodPics');

% Load data on the Pp's choices
choiceFile = fullfile(PTBParams.homepath,'SubjectData',num2str(PTBParams.subjid),...
    ['Data.', num2str(PTBParams.subjid), '.ChoiceTask.mat']);
ChoiceData = load(choiceFile);
ChoiceData = ChoiceData.Data;

%% Pick the trial to count
% Read in information about available foods. This information has a column
% containing the food stem (i.e. the text portion of the image files), as
% well as a number indicating the numerical ID of the maximum portion size
% currently available (e.g. If we have only 1 c. of Cheerios left, and the
% image name for that portion size is Cheerios_3_1c.jpg, the numerical ID
% contained in the portion availablility column would be 3.
[num, text] = xlsread(fullfile(PTBParams.homepath, 'Food Tracking.xlsx'));
foodNames = text(2:end,1);
foodNames(cellfun(@(x)~ischar(x),foodNames)) = [];
foodQuantities = num(:,1);

% Identify all trials for which we can deliver on their choice
availableTrials = [];
for i = 1:length(ChoiceData.Food)
   if ~isnan(ChoiceData.ChoiceRT{i}) && ChoiceData.Resp{i} <= 2
       % Any choice where they rejected the food is always available
       availableTrials = [availableTrials i];
   else
       if ~isnan(ChoiceData.ChoiceRT{i})
           % If they said YES, determine whether the portion size is available
           % First idenfity the food and the portion size on that trial
           fullFoodName = ChoiceData.Food{i};
           FoodStem = fullFoodName(1:(regexp(fullFoodName,'_','once') - 1));
           FoodAmount = str2num(fullFoodName(regexp(fullFoodName,'_','once') + 1));

           % Determine whether there's enough food, and if so, add this trial
           % to the list of available trials
           enoughFood = foodQuantities(strcmp(FoodStem,foodNames)) > FoodAmount;
           if enoughFood
               availableTrials = [availableTrials i];
           end
       end
   end
end

% Select one trial randomly from among all available trials
trialSelected = availableTrials(ceil(rand(1)*length(availableTrials)));
Food = ChoiceData.Food{trialSelected};
Insrx = ChoiceData.Instruction{trialSelected};
Choice = ChoiceData.Resp{trialSelected} > 2;

%% Show the results
% Display the instruction informing them that their food is about to be
% chosen
showInstruction(37,PTBParams,'RequiredKeys',{'RightArrow','right'});

% Display a blank screen, then reveal the trial components one at a time
Screen(PTBParams.win,'FillRect',PTBParams.black);
Screen(PTBParams.win,'Flip');
WaitSecs(1);

% Trial #
DrawFormattedText(PTBParams.win,['Trial # selected: ' num2str(trialSelected)],...
    'center',.2*PTBParams.ctr(2),PTBParams.white);

DrawFormattedText(PTBParams.win,['The trial looked like this:'],...
    'center',.35*PTBParams.ctr(2),PTBParams.white);
TrialRevealed = Screen(PTBParams.win,'Flip',[],1);
WaitSecs(1);

% Food and instruction on trial
[FoodPic, FoodPicSize] = makeTxtrFromImg(fullfile(PTBParams.foodPath,Food),...
    'JPG',PTBParams);
FoodPicPosition = findPicLoc(FoodPicSize, [.5, .6], PTBParams,...
    'ScreenPct', .35); 
Screen('DrawTexture',PTBParams.win, FoodPic, [], FoodPicPosition);
DrawFormattedText(PTBParams.win,Insrx,...
    'center',.5*PTBParams.ctr(2),PTBParams.white);
TrialRevealed = Screen(PTBParams.win,'Flip',[],1);
WaitSecs(1);

% Choice made on that trial
if Choice == 1
        foodText = 'You responded yes. You must now eat this food.';
else
        foodText = 'You responded no. You will NOT eat any food.';
end

DrawFormattedText(PTBParams.win,[foodText],...
    'center',1.85*PTBParams.ctr(2),PTBParams.white,55,[],[],1.75);

FoodOn = Screen(PTBParams.win,'Flip',TrialRevealed+1,1);

% Continue to display this outcome until the experiment presses both the
% 'c' and the 'o' keys (for COntinue)
collectResponse([],1,'c');
collectResponse([],1,'o');

%% End task and close all windows if called independently
endTime = datestr(now);
logData(pFile,1,endTime);

if isempty(varargin)
    close all; 
    Screen('CloseAll'); 
    ListenChar(1);
end