function runChoiceTask(varargin)
%-------------------------------------------------------------------------%
% Script for running a single subject through a choice task. Participants
% make choices in one of two conditions: Respond Naturally or Focus on
% Healthy Eating. These conditions are randomly interleaved in blocks of 10
% trials apiece, with order of foods randomized within each condition. 
% Participants will make their choice by indicating Strong No, Weak No, 
% Weak Yes, Strong Yes.
%
% Author: Cendri Hutcherson
% Last modified: 05-27-2018
%-------------------------------------------------------------------------%

%% Initial set up
% Allows two modes: called from main script, or called independently

studyID = 'ExampleStudy'; % this should match the study directory name
defaultSessionID = 'ChoiceTask';

if isempty(varargin)
    % If called independently, set up PsychToolBox, add the proper paths,
    % and then create a data file
    
    % Get paths to allow for independence across platforms
    pathToFile = mfilename('fullpath');
    homePath = pathToFile(1:(regexp(pathToFile,studyID) + length(studyID)));
    addpath(fullfile(homePath, 'PTBscripts'))
    addpath(fullfile(homePath, 'PTBscripts', 'PTBToolBox'))
    PTBParams = InitPTB(homePath,'DefaultSession',defaultSessionID);
else
    % If called from runStudy.m, the function will be supplied with the
    % PTBParams variable, which will be used to create a new data file for
    % this task component
    PTBParams = varargin{1};
    
    Data.subjid = PTBParams.subjid;
    Data.ssnid = defaultSessionID;
    Data.startTime = datestr(now);
 
    PTBParams.datafile = fullfile(PTBParams.homepath, 'SubjectData', ...
                         num2str(PTBParams.subjid), ...
                         ['Data.' num2str(PTBParams.subjid) '.' Data.ssnid '.mat']);
    save(PTBParams.datafile, 'Data')
end

pFile = PTBParams.datafile;

SessionClockStart = GetSecs();
logData(pFile,1,SessionClockStart);
PTBParams.SessionClockStart = SessionClockStart;

%% Set info about where to locate images and preload commonly used images
% All images are stored in a folder called 'ImageFiles' located inside the
% PTBscripts folder
imgpath = fullfile(PTBParams.homepath, 'PTBscripts', 'ImageFiles');

% Load in images that will display instructions for the
% 'Respond Naturally' condition and the 'Focus on Healthy Eating' condition
[PTBParams.NatInsrx, PTBParams.NatPicSize] = ...
    makeTxtrFromImg(fullfile(imgpath, 'NatInsrx.png'), 'PNG', PTBParams);

[PTBParams.RegInsrx, PTBParams.RegPicSize] = ...
    makeTxtrFromImg(fullfile(imgpath, 'HealthInsrx.jpg'), 'JPG', PTBParams); 

% Set key-value mapping (counterbalances right/left subject response order) 
% and load in proper choice key image
if mod(PTBParams.subjid, 2) == 1
    % If the subject number is odd, the response will be Strong Yes, Weak
    % Yes, Weak No, Strong No
    PTBParams.KeyOrder = {4 3 2 1};
    [PTBParams.ChoiceKeys, PTBParams.ChoiceKeysSize] = ...
        makeTxtrFromImg(fullfile(imgpath, 'ChoiceKeysRL.png'), 'PNG', PTBParams);
else
    % If the subject number is even, the response will be Strong No, Weak
    % No, Weak Yes, Strong Yes
    PTBParams.KeyOrder = {1 2 3 4};
    [PTBParams.ChoiceKeys, PTBParams.ChoiceKeysSize] = ...
        makeTxtrFromImg(fullfile(imgpath, 'ChoiceKeysLR.png'), 'PNG', PTBParams);

end

%% Create stimulus order
% This includes both the order of condition blocks, and the order of foods
% within those conditions.

% First, we will define the path to the folder where all the food images 
% are stored, and get the names of the food images to be displayed. The
% actual images themselves will be loaded in real time by the
% runChoiceTrial.m function
PTBParams.foodPath = fullfile(imgpath, 'FoodPics');

% Next, we will retrieve a list all of the food image files to be used in 
% the study, which we can simply define as the images located in the 
% FoodPics directory. In this example, there are 50 foods, with two 
% different portion sizes (small and large) each, for a total of 100 food 
% images. However, the script is written such that it will work flexibly 
% with however many foods as are located in the directory. The listing
% shown here uses the '*.jpg' formulation which ensures that only files of 
% the form someFoodName.jpg are listed
foodFiles = dir([PTBParams.foodPath filesep '*.jpg']);
foodNames = {foodFiles.name};

% Now we will define the order of stimulus presentation. What we want to do
% is to assign the two portion sizes of each food to only ONE of the two
% conditions (i.e., a small and a large portion of broccoli will be both
% randomly assigned to either the Respond Naturally or the Focus on Health
% condition. This means in later analyses we can better ask how focusing on
% health changes people's sensitivity to portion size
for i = 1:length(foodNames)
    FoodStem{i} = foodNames{i}(1:(regexp(foodNames{i},'_','once') - 1));
end

% Get only unique food names, then randomize those names for distribution
% to one of the two conditions. Assign both portion sizes of the food to
% one of the two conditions in alternating order
uniqueFoods = unique(FoodStem);
uniqueFoods = uniqueFoods(randperm(length(uniqueFoods)));

FoodsForNatural = {};
FoodsForHealthFocus = {};
for i = 1:length(uniqueFoods)
    twoPortions = foodNames(searchcell(foodNames,uniqueFoods{i},'contains'));
    if mod(i,2) == 0
        FoodsForNatural = [FoodsForNatural twoPortions];
    else
        FoodsForHealthFocus = [FoodsForHealthFocus twoPortions];
    end
end

% Finally, we will fully randomize the order of food presentation within
% each condition
FoodsForNatural = FoodsForNatural(randperm(length(FoodsForNatural)));
FoodsForHealthFocus = FoodsForHealthFocus(randperm(length(FoodsForHealthFocus)));

% Now, determine pseudorandom order of blocks (no more than 2 reps of any 
% given block type before switching to a different type
numBlocks = length(FoodsForNatural)/10;
BlockOrder = [];
for block = 1:numBlocks
    BlockOrder = [BlockOrder, randperm(2)];
end

% save information about the stimulus order to our datafile
logData(pFile,1,FoodsForNatural,FoodsForHealthFocus, BlockOrder);



%% Run the task!

% Loop over condition blocks
trial = 0;
for block = 1:length(BlockOrder) 
    % Define the instruction image to show at the beginning of the block,
    % and get the list of 10 foods to show, removing them from the list
    % after they've been retrieved
    if BlockOrder(block) == 1 % Respond Naturally
            InsrxPic = PTBParams.NatInsrx;
            InsrxSize = PTBParams.NatPicSize;
            Food = FoodsForNatural(1:10);
            FoodsForNatural(1:10) = [];
            Insrx = 'Respond Naturally';
    else % Focus on Healthy Eating
            InsrxPic = PTBParams.RegInsrx;
            InsrxSize = PTBParams.RegPicSize;
            Food = FoodsForHealthFocus(1:10);
            FoodsForHealthFocus(1:10) = [];
            Insrx = 'Focus on Healthiness';
    end
    
    % Display the block-level instruction for 5 seconds prior to beginning
    % the block of 10 trials
    Screen('DrawTexture',PTBParams.win,InsrxPic,[],...
        findPicLoc(InsrxSize,[.5,.5],PTBParams,'ScreenPct',1));
    Screen('Flip',PTBParams.win);
    WaitSecs(5);
    
    % Run the 10 trials in that block
    for t = 1:10
        % Augment the trial # by 1
        trial = trial + 1;   
        % Run the trial
        TrialData = runChoiceTrial(Food{t},Insrx,PTBParams);
        % Log the data
        logData(pFile, trial, TrialData);
    end
    
    % Give participants a break every 40 trials
    if mod(block,4) == 0
        Screen('FillRect',PTBParams.win,PTBParams.black);
        DrawFormattedText(PTBParams.win,['You may now take a break.\n'...
            'Whenever you are ready to continue, press any key.'], 'center',...
            'center',PTBParams.white,40);
        Screen('Flip',PTBParams.win);
        collectResponse;
    end
    
end % end of loop over blocks



%% End task and close all windows if called independently
endTime = datestr(now);
logData(pFile,1,endTime);

if isempty(varargin)
    close all; 
    Screen('CloseAll'); 
    ListenChar(1);
end