function TrialData = runChoiceTrial(FoodOnTrial, Instruction, PTBParams)
%-------------------------------------------------------------------------%
% function TrialData = runChoiceTrial(FoodOnTrial, Instruction, PTBParams)
%
% Usage: takes a string identifying a food picture (just the name, not the 
% full path, and an Instruction about how to regulate, and presents the
% food along with a set of choice keys (Strong No - Strong Yes)
%
% Trial structure:
%    1. Pre-stimulus fixation
%    2. Food presentation (max 4 sec) + choice
%    3. Choice feedback (250ms)
%    4. Post-stimulus fixation (ITI)
% 
%-------------------------------------------------------------------------%

ctr = PTBParams.ctr; % define the center of the screen

% Prepare the food image for display
[FoodPic, FoodPicSize] = ...
    makeTxtrFromImg(fullfile(PTBParams.foodPath,FoodOnTrial),...
    'JPG',PTBParams);

FoodPicPosition = findPicLoc(FoodPicSize, [.5, .45], PTBParams, ...
    'ScreenPct', .35);

% Prepare the response keys image for display
KeyPicPosition = findPicLoc(PTBParams.ChoiceKeysSize,[.5,.85],PTBParams, ...
    'ScreenPct',.4);

LateResp = 'NULL';
LateRT = NaN;

%% Display a fixation cross screen for 500ms
Screen('FillRect',PTBParams.win,PTBParams.black); 
Screen(PTBParams.win,'Flip');


%% Present food and collect response
% Draw the food
Screen('DrawTexture',PTBParams.win,FoodPic,[],FoodPicPosition);
% Draw the instruction cue at the top of the screen
DrawFormattedText(PTBParams.win, ... Screen
                  Instruction,... Instructional Cue
                  'center',.25*ctr(2),PTBParams.white);
% Draw the response keys at the bottom of the screen
Screen('DrawTexture',PTBParams.win,PTBParams.ChoiceKeys,[],KeyPicPosition);

% Display the food
FoodOn = Screen(PTBParams.win,'Flip',[],1);

% Collect a response of 1-4 with a deadline of 4 seconds, allowing for test
% mode
if PTBParams.subjid > 1000
    PTBParams.testMode = true;
end
[Resp, RT] = collectResponse(4,1,PTBParams.numKeys(1:4),PTBParams.KbDevice,PTBParams);
if ~strcmp(Resp,'NULL')
    Resp = str2double(Resp(1));
end

% Calculate RT relative to the onset time of the food
RT = RT - FoodOn;

%% Display the chosen response for 300 ms
rectSize = abs(KeyPicPosition([1,2]) - KeyPicPosition([3,4]));

if ~strcmp(RT,'NaN') && ~isnan(RT)
% Draw a red square around chosen option for 300ms
    Screen('FrameRect', PTBParams.win, [255,0,0], ...
           [KeyPicPosition(1) + (.25*(Resp - 1) + .05)*rectSize(1), ...
            KeyPicPosition(2), ...
            KeyPicPosition(1) + (.25*Resp - .05) *rectSize(1), ...
            KeyPicPosition(2) + .45*rectSize(2)],...
            5); % lineWidth
    ChoiceDisplayed = Screen('Flip',PTBParams.win);
end

%% Show fixation (or remind participant to respond in 4 secs)
if strcmp(RT,'NaN') || isnan(RT)
    % If participant doesn't respond within 4 seconds, remind them of the
    % time limit
    Screen(PTBParams.win,'FillRect',[0 0 0]);
    DrawFormattedText(PTBParams.win, '    TOO SLOW!\n\nYOUR CHOICE WILL BE DETERMINED AT RANDOM.', ...
                      'center','center', PTBParams.white);
    ChoiceDisplayed = Screen(PTBParams.win,'Flip');
    [LateResp, LateRT] = collectResponse(.95,0,PTBParams.numKeys(1:4));
    
    PostChoiceFixation = DrawFormattedText(PTBParams.win, '+', 'center','center', PTBParams.white);
    Screen('Flip', PTBParams.win,ChoiceDisplayed + 1,1);   
    LateRT = LateRT - FoodOn;
    if ~strcmp(LateResp,'NULL')
        LateResp = str2double(LateResp(1));
        LateResp = PTBParams.KeyOrder{LateResp}; % recode response by RL orientation
    end
else
    % Display fixation after choice
    DrawFormattedText(PTBParams.win, '+', 'center','center', PTBParams.white);
    PostChoiceFixation = Screen('Flip', PTBParams.win,ChoiceDisplayed + .25,1);

    Resp = PTBParams.KeyOrder{Resp}; % recode response by RL orientation
end

%% Present 1 second ITI
WaitSecs(1); 

%% Bundle up the data for logging with logData.m
TrialData.Food = FoodOnTrial;
TrialData.Resp = Resp;
TrialData.ChoiceRT  = RT;
TrialData.LateResp = LateResp;
TrialData.LateRT = LateRT;
TrialData.Instruction = Instruction;
TrialData.FoodOn = FoodOn - PTBParams.SessionClockStart;
TrialData.PostChoiceFixationOn = PostChoiceFixation - PTBParams.SessionClockStart;

% close the food texture (useful for avoiding memory issues)
Screen('Close',FoodPic);