function runInstructionTask(varargin)
%-------------------------------------------------------------------------%
% Script for running a single subject through instructions for the choice
% task. First introduce them to basic idea of making a choice. Then
% introduce them to the different attention/regulation conditions (Respond
% Naturally or Focus on Healthiness), then give them a quiz to make sure
% they understood the directions.
%
% Author: Cendri Hutcherson
% Last modified: 05-27-2018
%-------------------------------------------------------------------------%

%% Initial set up
% Allows two modes: called from main script, or called independently

studyID = 'ExampleStudy'; % this should match the study directory name
defaultSessionID = 'PracticeQuiz';

if isempty(varargin)
    % If called independently, set up PsychToolBox, add the proper paths,
    % and then create a data file
    
    % Get paths to allow for independence across platforms
    pathToFile = mfilename('fullpath');
    homePath = pathToFile(1:(regexp(pathToFile,studyID) + length(studyID)));
    addpath(fullfile(homePath, 'PTBscripts'))
    addpath(fullfile(homePath, 'PTBscripts', 'PTBToolBox'))
    PTBParams = InitPTB(homePath,'DefaultSession',defaultSessionID);
else
    % If called from runStudy.m, the function will be supplied with the
    % PTBParams variable, which will be used to create a new data file for
    % this task component
    PTBParams = varargin{1};
    
    Data.subjid = PTBParams.subjid;
    Data.ssnid = defaultSessionID;
    Data.startTime = datestr(now);
 
    PTBParams.datafile = fullfile(PTBParams.homepath, 'SubjectData', ...
                         num2str(PTBParams.subjid), ...
                         ['Data.' num2str(PTBParams.subjid) '.' Data.ssnid '.mat']);
    save(PTBParams.datafile, 'Data')
end

pFile = PTBParams.datafile;

SessionClockStart = GetSecs();
logData(pFile,1,SessionClockStart);
PTBParams.SessionClockStart = SessionClockStart;

% Initialize variables ---------------------------------------------------%  

% Set info about where to locate images and preload commonly used images
% All images are stored in a folder called 'ImageFiles' located inside the
% PTBscripts folder. Foods for the practice session are located inside the
% 'FoodPics' folder, in a subdirectory called 'PracticeFoodPics'
imgpath = fullfile(PTBParams.homepath, 'PTBscripts', 'ImageFiles');

% Load in images that will display instructions for the
% 'Respond Naturally' condition and the 'Focus on Healthy Eating' condition
[PTBParams.NatInsrx, PTBParams.NatPicSize] = ...
    makeTxtrFromImg(fullfile(imgpath, 'NatInsrx.png'), 'PNG', PTBParams);

[PTBParams.RegInsrx, PTBParams.RegPicSize] = ...
    makeTxtrFromImg(fullfile(imgpath, 'HealthInsrx.jpg'), 'JPG', PTBParams); 


PTBParams.foodPath = fullfile(imgpath, 'PracticeFoodPics');

% Set key-value mapping (counterbalances right/left subject response order) 
% and load in proper choice key image
if mod(PTBParams.subjid, 2) == 1
    % If the subject number is odd, the response will be Strong Yes, Weak
    % Yes, Weak No, Strong No
    PTBParams.KeyOrder = {4 3 2 1};
    [PTBParams.ChoiceKeys, PTBParams.ChoiceKeysSize] = ...
        makeTxtrFromImg(fullfile(imgpath, 'ChoiceKeysRL.png'), 'PNG', PTBParams);
else
    % If the subject number is even, the response will be Strong No, Weak
    % No, Weak Yes, Strong Yes
    PTBParams.KeyOrder = {1 2 3 4};
    [PTBParams.ChoiceKeys, PTBParams.ChoiceKeysSize] = ...
        makeTxtrFromImg(fullfile(imgpath, 'ChoiceKeysLR.png'), 'PNG', PTBParams);

end

% Give basic food choice instructions and 4 practice trials --------------%
% Implement a loop for slides 1-9 that allows the Pp to go back and forth
% among the instructions until they are ready to go on
insrx = 1;
while insrx >= 1 && insrx <= 9
    switch insrx
        case 1
            % If this is the first instruction slide, participants can only
            % go forward (right arrow)
            showInstruction(insrx,PTBParams,'RequiredKeys',{'RightArrow','right'});
            insrx = insrx + 1;
        case 3
           % If this is the 3rd instruction slide, determine whether to
           % show the slide with right-left or left-right ordering of the
           % response keys
           Resp = showInstruction(insrx,PTBParams,'RequiredKeys',...
            {'RightArrow','LeftArrow','right','left'});
            if isequal(Resp,'LeftArrow') || isequal(Resp,'left')
                % If left arrow, go back one slide
                insrx = insrx - 1;
            else
                if mod(PTBParams.subjid, 2) == 1
                    insrx = insrx + 2;
                else
                    insrx = insrx + 1;
                end
            end
        case 4
            Resp = showInstruction(insrx,PTBParams,'RequiredKeys',...
            {'RightArrow','LeftArrow','right','left'});
            if isequal(Resp,'LeftArrow') || isequal(Resp,'left')
                % If left arrow, go back one slide
                insrx = insrx - 1;
            else
                % Skip slide 5 (the slide showing other order
                insrx = insrx + 2;
            end
        case 5
            Resp = showInstruction(insrx,PTBParams,'RequiredKeys',...
            {'RightArrow','LeftArrow','right','left'});
            if isequal(Resp,'LeftArrow') || isequal(Resp,'left')
                % If left arrow, go back two slides (skipping the slide
                % showing the other order)
                insrx = insrx - 2;
            else
                insrx = insrx + 1;
            end  
        otherwise
        % Allow participants to go either forward by pressing the right
        % arrow or backwards by pressing the left arrow, advance or
        % decrement the slide by 1
        Resp = showInstruction(insrx,PTBParams,'RequiredKeys',...
            {'RightArrow','LeftArrow','right','left'});
        if isequal(Resp,'LeftArrow') || isequal(Resp,'left')
            insrx = insrx - 1;
        else
            insrx = insrx + 1;
        end
    end
end

% Run 4 trials of the Respond Naturally instructional condition with these 
% practice foods
FoodNames = {'WetBeanCurd_3_8pc.jpg','TeddyGrahams_2_15pc.jpg',...
             'Tostitos_4_15chips.jpg','Snickers_2_1Bar.jpg'};

for trial = randperm(4) 
    TrialData = runChoiceTrial(FoodNames{trial},'', PTBParams);
end

% Implement a loop for slides 10-18 that allows the Pp to go back and forth
% among the instructions until they are ready to go on. Instructs
% participants about the regulation tasks
while insrx >= 10 && insrx <= 18
    switch insrx
        case 10
            % If the first slide in the loop, only allow advance forward
            showInstruction(insrx,PTBParams,'RequiredKeys',{'RightArrow','right'});
            insrx = insrx + 1;
        otherwise
            Resp = showInstruction(insrx,PTBParams,'RequiredKeys',...
                {'RightArrow','LeftArrow','right','left'});
            if isequal(Resp,'LeftArrow') || isequal(Resp,'left')
                insrx = insrx - 1;
            else
                insrx = insrx + 1;
            end
    end
end

% Run 4 trials of the Health Focus instructional condition with these 
% practice foods
FoodNames = {'WetBeanCurd_3_8pc.jpg','TeddyGrahams_2_15pc.jpg',...
             'Tostitos_4_15chips.jpg','Snickers_2_1Bar.jpg'};

% Display the block-level instruction for 5 seconds prior to beginning
% the block of practice trials
Screen('DrawTexture',PTBParams.win,PTBParams.RegInsrx,[],...
    findPicLoc(PTBParams.RegPicSize,[.5,.5],PTBParams,'ScreenPct',1));
Screen('Flip',PTBParams.win);
WaitSecs(5);
    
for trial = randperm(4) 
    TrialData = runChoiceTrial(FoodNames{trial},'Focus on Healthiness', PTBParams);
end


% Run comprehension quiz ------------------------------------------------ %
% We will have the participants answer 4 questions. For each questions, a
% slide asking the questions and giving 3 possible responses will be shown.
% Following response, a feedback screen will be shown indicating either a
% correct or incorrect response and a written explanation of the correct 
% answer

% Give preamble slide to the quiz
showInstruction(insrx,PTBParams,'RequiredKeys',{'RightArrow','right'});
insrx = insrx + 1;

CorrectAnswers = [2, 3, 1, 3];

for q = 1:length(CorrectAnswers)
    QuizResp = showInstruction(insrx, PTBParams, ...
        'RequiredKeys',PTBParams.numKeys(1:3));
    QuizResp = QuizResp(1);
    
    CorrectResponse = str2num(QuizResp) == CorrectAnswers(q);
    % Print a display to the screen of whether the participant got it
    % correct. This can be used if you want to check and give explicit
    % verbal corrective feedback
    fprintf('Question %d: %d\n',q,CorrectResponse)
    
    if CorrectResponse
        showInstruction(insrx + 1, PTBParams,'RequiredKeys',{'RightArrow','right'});
    else
        showInstruction(insrx + 2, PTBParams,'RequiredKeys',{'RightArrow','right'});
    end
    
    % Save their answers, to be able to check for correct comprehension
    % later
    logData(pFile, q, QuizResp, CorrectResponse);
    
    insrx = insrx + 3; % advance to the next quiz question
end

% Finally, show the last instruction slide before beginning the actual
% choice trials
showInstruction(insrx, PTBParams,'RequiredKeys',{'RightArrow','right'});



%% End task and close all windows if called independently
endTime = datestr(now);
logData(pFile,1,endTime);

if isempty(varargin)
    close all; 
    Screen('CloseAll'); 
    ListenChar(1);
end
