function runRatingTask(varargin)
%-------------------------------------------------------------------------%
% Script for running a single subject through the subjective ratings task.
% Participants see all of the foods previously presented in the study and
% rate them for 3 quantities: overall liking, tastiness, and healthiness
%
% Author: Cendri Hutcherson
% Last modified: 05-27-2018
%-------------------------------------------------------------------------%

%% Initial set up
% Allows two modes: called from main script, or called independently

studyID = 'ExampleStudy'; % this should match the study directory name
defaultSessionID = 'AttributeRatings';

if isempty(varargin)
    % If called independently, set up PsychToolBox, add the proper paths,
    % and then create a data file    
    % Get paths to allow for independence across platforms
    pathToFile = mfilename('fullpath');
    homePath = pathToFile(1:(regexp(pathToFile,studyID) + length(studyID)));
    addpath(fullfile(homePath, 'PTBscripts'))
    addpath(fullfile(homePath, 'PTBscripts', 'PTBToolBox'))
    PTBParams = InitPTB(homePath,'DefaultSession',defaultSessionID);
else
    % If called from runStudy.m, the function will be supplied with the
    % PTBParams variable, which will be used to create a new data file for
    % this task component
    PTBParams = varargin{1};
    
    Data.subjid = PTBParams.subjid;
    Data.ssnid = defaultSessionID;
    Data.startTime = datestr(now);
 
    PTBParams.datafile = fullfile(PTBParams.homepath, 'SubjectData', ...
                         num2str(PTBParams.subjid), ...
                         ['Data.' num2str(PTBParams.subjid) '.' Data.ssnid '.mat']);
    save(PTBParams.datafile, 'Data')
end

pFile = PTBParams.datafile;

SessionClockStart = GetSecs();
logData(pFile,1,SessionClockStart);
PTBParams.SessionClockStart = SessionClockStart;

%% Set info about where to locate images
% All images are stored in a folder called 'ImageFiles' located inside the
% PTBscripts folder
imgpath = fullfile(PTBParams.homepath, 'PTBscripts', 'ImageFiles');

% First, we will define the path to the folder where all the food images 
% are stored, and get the names of the food images to be displayed. The
% actual images themselves will be loaded in real time by the
% runRatingTrial.m function
PTBParams.foodPath = fullfile(imgpath, 'FoodPics');

% Next, we will retrieve a list all of the food image files to be rated, 
% which we can simply define as the images located in the 
% FoodPics directory. In this example, there are 50 foods, with two 
% different portion sizes (small and large) each, for a total of 100 food 
% images. However, the script is written such that it will work flexibly 
% with however many foods as are located in the directory. The listing
% shown here uses the '*.jpg' formulation which ensures that only files of 
% the form someFoodName.jpg are listed
foodFiles = dir([PTBParams.foodPath filesep '*.jpg']);
PTBParams.FoodNames = {foodFiles.name};

%% Collect overall liking ratings --------------------------------------- %
% First, we will show instructions to the Pp about making overall liking
% ratings. These ratings always come first.
insrx = 33;
while insrx >=33 && insrx <= 34
    if insrx == 33
        showInstruction(33,PTBParams,'RequiredKeys',{'RightArrow','right'});
        insrx = insrx + 1;
    else
        Resp = showInstruction(insrx,PTBParams,'RequiredKeys',{'RightArrow','LeftArrow','right','left'});
        if strcmp(Resp,'LeftArrow') || strcmp(Resp,'left')
            insrx = insrx - 1;
        else
            insrx = insrx + 1;
        end
    end
end

% Define the image that shows keys for making the liking ratings, then
% show the foods from the study in randomized order, collecting the ratings
[PTBParams.RateKeys, PTBParams.RateKeysSize] = ...
                makeTxtrFromImg(fullfile(imgpath, 'LikingRatingKeys.png'),...
                'PNG', PTBParams);
trial = 1;
for food = randperm(length(PTBParams.FoodNames))
    TrialData = runRatingTrial(food, PTBParams);
    TrialData.Attribute = 'Liking';
    logData(pFile, trial, TrialData)
    trial = trial + 1;
end

%% Run the taste and health attribute ratings components. 
% Collect all ratings for one attribute, then all ratings for the other
% attribute. The order is random across participants.
RatingOrder = {'Health', 'Taste'};
RatingOrder = RatingOrder(randperm(length(RatingOrder)));
logData(pFile,1,RatingOrder);

% Loop over rounds of attribute ratings. Note that this code can be
% flexibly updated if you want to collect additional attribute ratings by
% simply defining additional cases
for r = 1:length(RatingOrder)
    switch RatingOrder{r}
        case 'Taste'
            % This slide contains the taste rating instructions
            insrx = 35;
            % Load in picture of taste rating keys
            [PTBParams.RateKeys, PTBParams.RateKeysSize] = ...
                makeTxtrFromImg(fullfile(imgpath, 'TasteRatingKeys.png'),...
                'PNG', PTBParams);
            
        case 'Health'
            % This slide contains the health rating instructions
            insrx = 36;
            % Load in pictures of health rating keys
            [PTBParams.RateKeys, PTBParams.RateKeysSize] = ...
                makeTxtrFromImg(fullfile(imgpath, 'HealthRatingKeys.png'),...
                'PNG', PTBParams);
    end
    
    % Display the appropriate instruction slide
    showInstruction(insrx, PTBParams,'RequiredKeys',{'RightArrow','right'});
    
    % Collect the ratings. These will be appended to the already-collected
    % Liking ratings
    for food = randperm(length(PTBParams.FoodNames))
        TrialData = runRatingTrial(food, PTBParams);
        TrialData.Attribute = RatingOrder{r};
        logData(pFile, trial, TrialData)
        trial = trial + 1;
    end

end
    
%% End task and close all windows if called independently
endTime = datestr(now);
logData(pFile,1,endTime);

if isempty(varargin)
    close all; 
    Screen('CloseAll'); 
    ListenChar(1);
end